package com.dz.openapi.demo.staff;


import com.dz.openapi.core.OpenAPIResponse;
import com.dz.openapi.demo.BaseWebTest;
import com.dz.openapi.demo.OpenAPIConstants;
import com.dz.openapi.demo.staff.model.DepartmentTree;
import com.dz.openapi.demo.staff.model.DepartmentTreeResponse;
import com.dz.openapi.demo.staff.model.RoleInfo;
import com.dz.openapi.demo.staff.model.RoleInfosResponse;
import com.dz.openapi.demo.staff.model.UserInfo;
import com.dz.openapi.demo.staff.model.UserInfoDTOExt;
import com.dz.openapi.demo.staff.model.UserInfosResponse;

import org.junit.jupiter.api.Test;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.util.UriComponentsBuilder;

import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Random;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class StaffTest extends BaseWebTest {
    private SimpleDateFormat nameDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");

    @Test
    public void staffTest() {

        // 查询租户部门列表
        DepartmentTree departmentTree = queryDepartmentTree();

        // 查询角色列表
        List<RoleInfo> roleInfos = queryRoleInfos();

        // 添加坐席
        addStaff(new Long[]{departmentTree.getId()}, new Long[]{roleInfos.get(0).getRoleId()});

        // 查询全部坐席列表
        List<UserInfoDTOExt> userInfoDTOExts = queryStaffList();
        // 获取最新创建的坐席
        UserInfoDTOExt newUserInfo = null;
        for (UserInfoDTOExt userInfoDTOExt : userInfoDTOExts) {
            if (newUserInfo == null || userInfoDTOExt.getId() > newUserInfo.getId()) {
                newUserInfo = userInfoDTOExt;
            }
        }

        // 更新坐席
        updateStaff(newUserInfo);

        // 重置密码
        resetPwd(newUserInfo);

        // 删除坐席
        deleteStaff(newUserInfo.getId());
    }

    /**
     * 查询租户部门列表
     */
    protected DepartmentTree queryDepartmentTree() {
        DepartmentTreeResponse departmentTreeResponse = restTemplate.getForObject(
                config.getAPIUrl(OpenAPIConstants.STAFF.DEPARTMENT_LIST), DepartmentTreeResponse.class);
        assertResultAndLog(departmentTreeResponse);
        return departmentTreeResponse.getData();
    }

    /**
     * 查询角色列表
     */
    protected List<RoleInfo> queryRoleInfos() {
        MultiValueMap<String, String> queryParamMap = new LinkedMultiValueMap<>();
        queryParamMap.add("roleName", "");
        UriComponentsBuilder queryBuilder = UriComponentsBuilder.fromHttpUrl(config.getAPIUrl(OpenAPIConstants.STAFF.ROLE_LIST)).queryParams(queryParamMap);
        RoleInfosResponse roleInfosResponse = restTemplate.getForObject(queryBuilder.toUriString(), RoleInfosResponse.class);
        assertResultAndLog(roleInfosResponse);
        return roleInfosResponse.getData();
    }

    /**
     * 查询坐席列表
     */
    protected List<UserInfoDTOExt> queryStaffList(String... userIds) {
        MultiValueMap<String, String> queryParamMap = new LinkedMultiValueMap<>();
        if (userIds.length > 0) {
            for (String userId : userIds) {
                queryParamMap.add("userIds", userId);
            }
        }
        UriComponentsBuilder queryBuilder = UriComponentsBuilder.fromHttpUrl(config.getAPIUrl(OpenAPIConstants.STAFF.STAFF_LIST)).queryParams(queryParamMap);
        UserInfosResponse userInfosResponse = restTemplate.getForObject(queryBuilder.toUriString(), UserInfosResponse.class);
        assertResultAndLog(userInfosResponse);
        return userInfosResponse.getData();
    }

    /**
     * 添加坐席
     */
    protected UserInfo addStaff(Long[] departmentIds, Long[] roleIds) {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);

        String timeMillis = String.valueOf(System.currentTimeMillis());
        UserInfo userInfo = new UserInfo();
        userInfo.setEmail(timeMillis + "@dzdemo.com");
        userInfo.setEmpno(timeMillis);
        userInfo.setPhone(timeMillis.substring(0, 11));
        userInfo.setName("OpenAPIDemo" + new Random().nextInt(10000));
        userInfo.setDepartmentIds(departmentIds);
        userInfo.setRoleIds(roleIds);

        HttpEntity<UserInfo> sendRequest = new HttpEntity<>(userInfo, headers);
        OpenAPIResponse response = restTemplate.exchange(
                config.getAPIUrl(OpenAPIConstants.STAFF.STAFF_ADD), HttpMethod.POST, sendRequest, OpenAPIResponse.class).getBody();
        assertResultAndLog(response);
        return userInfo;
    }

    /**
     * 添加坐席
     */
    protected void updateStaff(UserInfoDTOExt userInfoDTOExt) {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);

        String timeMillis = String.valueOf(System.currentTimeMillis());
        UserInfo userInfo = new UserInfo();
        userInfo.setId(userInfoDTOExt.getId());
        userInfo.setEmail(userInfoDTOExt.getEmail());
        userInfo.setEmpno(timeMillis);
        userInfo.setPhone(userInfoDTOExt.getPhone());
        userInfo.setName("OpenAPIDemo" + new Random().nextInt(10000));
        userInfo.setDepartmentIds(new Long[]{userInfoDTOExt.getDepartments().get(0).getId()});
        userInfo.setRoleIds(new Long[]{userInfoDTOExt.getRoles().get(0).getId()});

        HttpEntity<UserInfo> sendRequest = new HttpEntity<>(userInfo, headers);
        OpenAPIResponse response = restTemplate.exchange(
                config.getAPIUrl(OpenAPIConstants.STAFF.STAFF_UPDATE), HttpMethod.PUT, sendRequest, OpenAPIResponse.class).getBody();
        assertResultAndLog(response);
    }

    /**
     * 重置密码
     */
    protected void resetPwd(UserInfoDTOExt userInfoDTOExt) {
        MultiValueMap<String, String> queryParamMap = new LinkedMultiValueMap<>();
        queryParamMap.add("email", userInfoDTOExt.getEmail());
        UriComponentsBuilder queryBuilder = UriComponentsBuilder.fromHttpUrl(config.getAPIUrl(OpenAPIConstants.STAFF.STAFF_PWD_RESET)).queryParams(queryParamMap);

        HttpEntity sendRequest = new HttpEntity(null);
        OpenAPIResponse response = restTemplate.exchange(
                queryBuilder.toUriString(), HttpMethod.PUT, sendRequest, OpenAPIResponse.class).getBody();
        assertResultAndLog(response);
    }

    /**
     * 删除坐席
     */
    protected void deleteStaff(Long userId) {
        MultiValueMap<String, String> queryParamMap = new LinkedMultiValueMap<>();
        queryParamMap.add("id", String.valueOf(userId));
        UriComponentsBuilder queryBuilder = UriComponentsBuilder.fromHttpUrl(config.getAPIUrl(OpenAPIConstants.STAFF.STAFF_DELETE)).queryParams(queryParamMap);

        HttpEntity sendRequest = new HttpEntity(null);
        OpenAPIResponse response = restTemplate.exchange(
                queryBuilder.toUriString(), HttpMethod.DELETE, sendRequest, OpenAPIResponse.class).getBody();
        assertResultAndLog(response);
    }
}
